from .commission import (
    AccountTypeRoleCommissionFactory,
    CommissionFactory,
    CommissionRoleFactory,
    PortfolioRoleCommissionFactory,
    CommissionExclusionRuleFactory,
    CommissionTypeFactory,
)
from .rebate import RebateFactory



from django.contrib import admin
from wbcrm.admin.accounts import AccountModelAdmin as BaseAccountModelAdmin
from wbcrm.models.accounts import Account

from wbcommission.models.rebate import manage_rebate_as_task

from .commission import CommissionTabularInline

admin.site.unregister(Account)


@admin.register(Account)
class AccountModelAdmin(BaseAccountModelAdmin):
    def make_rebates(self, request, queryset):
        for account in queryset:
            manage_rebate_as_task.delay(account.id)

    def get_assets_under_management(self, request, queryset):
        for account in queryset:
            account.get_assets_under_management()

    actions = list(BaseAccountModelAdmin.actions) + [make_rebates, get_assets_under_management]
    inlines = BaseAccountModelAdmin.inlines + [CommissionTabularInline]



from wbcore import filters as wb_filters
from wbcore.contrib.directory.models import Company
from wbcore.filters.defaults import current_quarter_date_range
from wbcore.pandas.filterset import PandasFilterSetMixin
from wbfdm.models import ClassificationGroup
from wbfdm.preferences import get_default_classification_group
from wbportfolio.filters.transactions.claim import CommissionBaseFilterSet
from wbportfolio.models import Product

from wbcommission.models import Rebate
from wbcommission.models.rebate import RebateGroupbyChoice


class RebateDateFilter(CommissionBaseFilterSet):
    date = wb_filters.DateRangeFilter(
        method=wb_filters.DateRangeFilter.base_date_range_filter_method,
        label="Date Range",
        required=True,
        clearable=False,
        initial=current_quarter_date_range,
    )

    class Meta:
        model = Rebate
        fields = {
            "date": ["exact"],
            "account": ["exact"],
            "product": ["exact"],
            "recipient": ["exact"],
            "commission_type": ["exact"],
        }


class RebateGroupByFilter(PandasFilterSetMixin, RebateDateFilter):
    group_by = wb_filters.ChoiceFilter(
        label="Group By",
        choices=RebateGroupbyChoice.choices(),
        initial=RebateGroupbyChoice.ACCOUNT.name,  # typing: ignore
        method=lambda queryset, label, value: queryset,
        clearable=False,
        required=True,
    )
    groupby_classification_group = wb_filters.ModelChoiceFilter(
        initial=lambda k, v, f: get_default_classification_group().id,
        method=lambda queryset, label, value: queryset,
        label="Group by Classification Group",
        queryset=ClassificationGroup.objects.all(),
        endpoint=ClassificationGroup.get_representation_endpoint(),
        value_key=ClassificationGroup.get_representation_value_key(),
        label_key=ClassificationGroup.get_representation_label_key(),
    )

    class Meta:
        model = Rebate
        fields = {"recipient": ["exact"]}
        df_fields = {
            "management__gte": wb_filters.NumberFilter(
                label="Management Fees (USD)", lookup_expr="gte", field_name="management"
            ),
            "management__lte": wb_filters.NumberFilter(
                label="Management Fees (USD)", lookup_expr="lte", field_name="management"
            ),
            "performance__gte": wb_filters.NumberFilter(
                label="Performance Fees (USD)", lookup_expr="gte", field_name="performance"
            ),
            "performance__lte": wb_filters.NumberFilter(
                label="Performance Fees (USD)", lookup_expr="lte", field_name="performance"
            ),
            "total_rebate__gte": wb_filters.NumberFilter(
                label="Total Fees (USD)", lookup_expr="gte", field_name="total_rebate"
            ),
            "total_rebate__lte": wb_filters.NumberFilter(
                label="Total Fees (USD)", lookup_expr="lte", field_name="total_rebate"
            ),
        }


class CustomerRebateGroupByFilter(RebateGroupByFilter):
    product = wb_filters.ModelChoiceFilter(
        label="Product",
        queryset=Product.objects.all(),
        endpoint=Product.get_representation_endpoint(),
        value_key=Product.get_representation_value_key(),
        label_key="{{title}} {{currency_repr}} - {{isin}}",
    )

    class Meta:
        model = Rebate
        fields = {
            # "price_start" : ["gte", "exact", "lte"],
            # "price_end" : ["gte", "exact", "lte"],
            "account": ["exact"],
            "product": ["exact"],
            "recipient": ["exact"],
        }


class RebateMarginalityFilter(PandasFilterSetMixin, wb_filters.FilterSet):
    date = wb_filters.DateRangeFilter(
        label="Date Range",
        method=wb_filters.DateRangeFilter.base_date_range_filter_method,
        required=True,
        clearable=False,
        initial=current_quarter_date_range,
    )

    bank = wb_filters.ModelMultipleChoiceFilter(
        label="Bank",
        queryset=Company.objects.all(),
        endpoint=Company.get_representation_endpoint(),
        filter_params={"bank_product": True},
        value_key=Company.get_representation_value_key(),
        label_key=Company.get_representation_label_key(),
        method="filter_bank",
    )

    def filter_bank(self, queryset, name, value):
        if value:
            return queryset.filter(bank__in=value)
        return queryset

    class Meta:
        model = Product
        fields = {}
        df_fields = {
            "management_fees__gte": wb_filters.NumberFilter(
                field_name="management_fees", lookup_expr="gte", label="Sum Management Fees"
            ),
            "management_rebates__gte": wb_filters.NumberFilter(
                field_name="management_rebates", lookup_expr="gte", label="Sum Management Rebates"
            ),
            "management_marginality__gte": wb_filters.NumberFilter(
                field_name="management_marginality", lookup_expr="gte", label="Marginality Management"
            ),
            "performance_fees__gte": wb_filters.NumberFilter(
                field_name="performance_fees", lookup_expr="gte", label="Sum Performance Fees"
            ),
            "performance_rebates__gte": wb_filters.NumberFilter(
                field_name="performance_rebates", lookup_expr="gte", label="Sum Performance Rebates"
            ),
            "performance_marginality__gte": wb_filters.NumberFilter(
                field_name="performance_marginality", lookup_expr="gte", label="Marginality Performance"
            ),
            "total_fees__gte": wb_filters.NumberFilter(field_name="total_fees", lookup_expr="gte", label="Total Fees"),
            "total_rebates__gte": wb_filters.NumberFilter(
                field_name="total_rebates", lookup_expr="gte", label="Total Rebates"
            ),
            "total_marginality_percent__gte": wb_filters.NumberFilter(
                field_name="total_marginality_percent", lookup_expr="gte", label="Total Marginality"
            ),
            "total_fees_usd__gte": wb_filters.NumberFilter(
                field_name="total_fees_usd", lookup_expr="gte", label="Total Fees"
            ),
            "total_rebates_usd__gte": wb_filters.NumberFilter(
                field_name="total_rebates_usd", lookup_expr="gte", label="Total Rebates"
            ),
            "management_fees__lte": wb_filters.NumberFilter(
                field_name="management_fees", lookup_expr="lte", label="Sum Management Fees"
            ),
            "management_rebates__lte": wb_filters.NumberFilter(
                field_name="management_rebates", lookup_expr="lte", label="Sum Management Rebates"
            ),
            "management_marginality__lte": wb_filters.NumberFilter(
                field_name="management_marginality", lookup_expr="lte", label="Marginality Management"
            ),
            "performance_fees__lte": wb_filters.NumberFilter(
                field_name="performance_fees", lookup_expr="lte", label="Sum Performance Fees"
            ),
            "performance_rebates__lte": wb_filters.NumberFilter(
                field_name="performance_rebates", lookup_expr="lte", label="Sum Performance Rebates"
            ),
            "performance_marginality__lte": wb_filters.NumberFilter(
                field_name="performance_marginality", lookup_expr="lte", label="Marginality Performance"
            ),
            "total_fees__lte": wb_filters.NumberFilter(field_name="total_fees", lookup_expr="lte", label="Total Fees"),
            "total_rebates__lte": wb_filters.NumberFilter(
                field_name="total_rebates", lookup_expr="lte", label="Total Rebates"
            ),
            "total_marginality_percent__lte": wb_filters.NumberFilter(
                field_name="total_marginality_percent", lookup_expr="lte", label="Total Marginality"
            ),
            "total_fees_usd__lte": wb_filters.NumberFilter(
                field_name="total_fees_usd", lookup_expr="lte", label="Total Fees"
            ),
            "total_rebates_usd__lte": wb_filters.NumberFilter(
                field_name="total_rebates_usd", lookup_expr="lte", label="Total Rebates"
            ),
        }



import pytest
from wbcrm.factories import AccountFactory

from wbcommission.factories import CommissionTypeFactory
from wbcommission.models.account_service import AccountRebateManager


class AccountManagerFixture:
    @pytest.fixture()
    def performance_account_manager(self):
        commission_type = CommissionTypeFactory.create(name="PERFORMANCE")
        account_manager = AccountRebateManager(AccountFactory.create(), commission_type.key)
        account_manager.initialize()

        return account_manager

    @pytest.fixture()
    def management_account_manager(self):
        commission_type = CommissionTypeFactory.create(name="MANAGEMENT")
        account_manager = AccountRebateManager(AccountFactory.create(), commission_type.key)
        account_manager.initialize()

        return account_manager



import factory

from wbcommission.models import Rebate


class RebateFactory(factory.django.DjangoModelFactory):
    date = factory.Faker("date_object")

    commission = factory.SubFactory("wbcommission.factories.CommissionFactory")
    account = factory.LazyAttribute(lambda o: o.commission.account)
    commission_type = factory.LazyAttribute(lambda o: o.commission.commission_type)
    product = factory.SubFactory("wbportfolio.factories.ProductFactory")
    recipient = factory.SubFactory("wbcore.contrib.directory.factories.entries.PersonFactory")
    value = factory.Faker("pydecimal", positive=True, max_value=1000000, right_digits=4)

    class Meta:
        model = Rebate



# Generated by Django 4.1.10 on 2023-09-06 14:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0005_account_accountrole_accountroletype_and_more"),
        ("wbcommission", "0002_commissionrule_remove_accountcustomer_account_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="commission",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="account_commissions",
                to="wbcrm.account",
                verbose_name="Account",
            ),
        ),
    ]
